/**
    FREEZER - A tray icon for Deezer

    Copyright (C) 2007 Amos Brocco <amos.brocco@unifr.ch>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tray;
import org.eclipse.swt.widgets.TrayItem;

public class Freezer {

	private Display display;
	private Tray tray;
	private TrayItem trayItem;
	private Image trayIcon;
	private Menu trayMenu;
	private Menu playlistMenu;
	private Browser browser;
	private Shell shell;
	private Shell trayPopup;
	private HashMap<String, String> playlistsPaths;
	private String currentPlaylist = null;
	private String baseTooltip = "Freezer - %s";
	private String versionString = "0.02";
	private String deezerURL = "<object width=\"300\" height=\"250\"><param name=\"movie\" value=\"http://www.deezer.com/" +
					"embed/player?pid=%s&ap=1&ln=en&sl=1\"></param><param name=\"allowFullScreen\" value=\"true\">" +
					"</param><param name=\"allowscriptaccess\" value=\"always\"></param><embed src=\"http://www.deezer." +
					"com/embed/player?pid=%s&ap=1&ln=en&sl=1\" type=\"application/x-shockwave-flash\" allowscript" +
					"access=\"always\" allowfullscreen=\"true\" width=\"300\" height=\"250\"></embed></object>";
	private Preferences preferences;
	private String prefPlaylistKey = "freezer::playlists";
	private PlaylistManager p;
	private int WIDTH=315;
	private int HEIGHT=270;

	private String getToolTip() {
		if (currentPlaylist == null)
			return String.format(baseTooltip, "No playlist");
		else
			return String.format(baseTooltip, currentPlaylist);
	}

	private boolean browserActive() {
		return !browser.getUrl().equals("");
	}
	
	private Point adjustLocation(Point cursorLocation) {
		int w, h;
		int wshift = 5;
		int hshift = 5;
		w = cursorLocation.x;
		h = cursorLocation.y;
		
		if (h > 200) {
			hshift = -hshift;
		}
		
		/* Remove 10 to stay away from borders */
		int screenWidth = Display.getCurrent().getClientArea().width - 10;
		int screenHeight = Display.getCurrent().getClientArea().height - 10;
		
		if ((cursorLocation.y+HEIGHT) > (screenHeight + hshift)) {
			h =  screenHeight - HEIGHT;
		}
		if ((cursorLocation.x+WIDTH) > (screenWidth - wshift)) {
			w =  screenWidth - WIDTH;
		}
		if (cursorLocation.y < 0) {
			h =  0;
		}
		if (cursorLocation.x < 0) {
			w =  0;
		}
		w = w + wshift;
		h = h + hshift;
		
		return new Point(w,h);
	}

	private void updatePlaylistMenu() {
		/* Clear the menu */
		int count = playlistMenu.getItemCount();
		for (int i = 0; i < count; i++) {
			MenuItem item = playlistMenu.getItem(0);
			item.dispose();
		}

		Iterator<String> pliter = playlistsPaths.keySet().iterator();
		while (pliter.hasNext()) {
			String key = pliter.next();
			MenuItem plitem = new MenuItem(playlistMenu, SWT.PUSH);
			plitem.setText(key);
			plitem.setData(key);
			plitem.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					MenuItem source = (MenuItem) event.widget;
					changePlaylist((String) source.getData());
				}
			});
		}
	}

	private void showManagePlaylists() {
		p = new PlaylistManager(this, display);
		p.populate(playlistsPaths.keySet());
		p.show();
	}

	private void showAbout() {
		MessageBox aboutBox = new MessageBox(shell, SWT.ICON_INFORMATION
				| SWT.OK);
		aboutBox.setText("About Freezer");
		aboutBox
				.setMessage(String
						.format(
								"Freezer %s\n\n"
										+ "A tray icon for Deezer\n\n"
										+ "Copyright (C) 2007 Amos Brocco <amos.brocco@unifr.ch>\n\n"
										+ "This program is free software: you can redistribute it and/or modify "
										+ "it under the terms of the GNU General Public License as published by "
										+ "the Free Software Foundation, either version 3 of the License, or "
										+ "(at your option) any later version.\n"
										+ "\n"
										+ "This program is distributed in the hope that it will be useful, "
										+ "but WITHOUT ANY WARRANTY; without even the implied warranty of "
										+ "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
										+ "GNU General Public License for more details.\n"
										+ "\n"
										+ "You should have received a copy of the GNU General Public License "
										+ "along with this program.  If not, see \n<http://www.gnu.org/licenses/>.\n",
								versionString));
		aboutBox.open();
	}  
	
	private void playPause(Event ev) {
		if (!browserActive()) return;
		
		Point p = display.getCursorLocation();

		final int oX = p.x;
		final int oY = p.y;	
		
		trayPopup.getDisplay().syncExec(new Runnable(){
			public void run() {
				trayPopup.setLocation(oX,oY);
				trayPopup.setVisible(true);
			}
		});
		
		trayPopup.getDisplay().syncExec(new Runnable(){
			public void run() {
				Event event = new Event();
				event.type = SWT.MouseMove;
				event.x = trayPopup.toDisplay(158, 115).x;
				event.y = trayPopup.toDisplay(158, 115).y;
				trayPopup.getDisplay().post(event);
	
				event = new Event();
				event.type = SWT.MouseDown;
				event.button = 1;
				trayPopup.getDisplay().post(event);
	
				event = new Event();
				event.type = SWT.MouseUp;
				event.button = 1;
				trayPopup.getDisplay().post(event);
			
				trayPopup.getDisplay().asyncExec(new Runnable(){
					public void run() {
						try {
							Thread.sleep(2500);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
						trayPopup.setVisible(false);
					}
				});
				
				event = new Event();
				event.type = SWT.MouseMove;
				event.x = oX;
				event.y = oY;
				trayPopup.getDisplay().post(event);	
			}
			});	
	}
	
	private void nextSong(Event ev) {
		if (!browserActive()) return;
		
		Point p = display.getCursorLocation();

		final int oX = p.x;
		final int oY = p.y;	
		
		trayPopup.getDisplay().syncExec(new Runnable(){
			public void run() {
				trayPopup.setLocation(oX,oY);
				trayPopup.setVisible(true);
			}
		});
		
		trayPopup.getDisplay().syncExec(new Runnable(){
			public void run() {
				Event event = new Event();
				event.type = SWT.MouseMove;
				event.x = trayPopup.toDisplay(198, 115).x;
				event.y = trayPopup.toDisplay(198, 115).y;
				trayPopup.getDisplay().post(event);
	
				event = new Event();
				event.type = SWT.MouseDown;
				event.button = 1;
				trayPopup.getDisplay().post(event);
	
				event = new Event();
				event.type = SWT.MouseUp;
				event.button = 1;
				trayPopup.getDisplay().post(event);
			
				trayPopup.getDisplay().asyncExec(new Runnable(){
					public void run() {
						try {
							Thread.sleep(2500);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
						trayPopup.setVisible(false);
					}
				});
				
				event = new Event();
				event.type = SWT.MouseMove;
				event.x = oX;
				event.y = oY;
				trayPopup.getDisplay().post(event);	
			}
			});	
	}

	public void initTray() {
		shell = new Shell(display);
		trayPopup = new Shell(shell, SWT.NO_TRIM | SWT.ON_TOP);
		trayPopup.setLayout(new FillLayout());
		trayPopup.setSize(WIDTH, HEIGHT);
		trayPopup.setVisible(false);
		try {
			browser = new Browser(trayPopup, SWT.NONE);
		} catch (SWTError e) {
			System.out.println("Could not instantiate Browser: "
					+ e.getMessage());
			return;
		}

		trayMenu = new Menu(shell, SWT.POP_UP);
		MenuItem playpause = new MenuItem(trayMenu, SWT.PUSH);
		playpause.setText("&Play/Pause");
		playpause.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				playPause(event);
			}
		});
		
		MenuItem nextSong = new MenuItem(trayMenu, SWT.PUSH);
		nextSong.setText("&Next song");
		nextSong.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				nextSong(event);
			}
		});
		
		
		new MenuItem(trayMenu, SWT.SEPARATOR);
		MenuItem plitem = new MenuItem(trayMenu, SWT.CASCADE);
		plitem.setText("&Playlists...");
		playlistMenu = new Menu(trayMenu);
		plitem.setMenu(playlistMenu);
		trayMenu.setDefaultItem(plitem);
		new MenuItem(trayMenu, SWT.SEPARATOR);
		MenuItem plmanage = new MenuItem(trayMenu, SWT.PUSH);
		plmanage.setText("&Manage playlists");
		plmanage.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				showManagePlaylists();
			}
		});
		new MenuItem(trayMenu, SWT.SEPARATOR);
		MenuItem about = new MenuItem(trayMenu, SWT.PUSH);
		about.setText("&About");
		about.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				showAbout();
			}
		});
		new MenuItem(trayMenu, SWT.SEPARATOR);
		MenuItem closeApp = new MenuItem(trayMenu, SWT.PUSH);
		closeApp.setText("E&xit");
		closeApp.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				browser.stop();
				browser.dispose();
				shell.dispose();
			}
		});
		trayIcon = new Image(shell.getDisplay(), Freezer.class.getResourceAsStream(
	      "freezer.png"));
		tray = display.getSystemTray();
		if (tray == null) {
			System.out.println("Freezer: tray not available");
		} else {
			trayItem = new TrayItem(tray, SWT.NONE);
			trayItem.setToolTipText(getToolTip());
			trayItem.setImage(trayIcon);
			trayItem.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					if (browserActive()) {
						trayPopup.setLocation(adjustLocation(tray.getDisplay()
								.getCursorLocation()));
						trayPopup.setVisible(!trayPopup.getVisible());
					} else {
						trayPopup.setVisible(false);
						if(playlistsPaths.size() == 0)
							showManagePlaylists();
					}
				}
			});
			trayItem.addListener(SWT.MenuDetect, new Listener() {
				public void handleEvent(Event event) {
					trayMenu.setVisible(true);
				}
			});
		}
	}

	private void changePlaylist(String key) {
		String path = playlistsPaths.get(key);
		if (path != null) {
			currentPlaylist = key;
			trayItem.setToolTipText(getToolTip());
			browser.setText(String.format(deezerURL, path, path));
		}
	}

	public void savePlaylists() {
		preferences.remove("playlists");
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		Serializer ser = new Serializer(out);
		try {
			ser.print(playlistsPaths);
			String value = out.toString();
			preferences.put(prefPlaylistKey, value);
			preferences.sync();
		} catch (Exception e) {
			e.printStackTrace();
			savePrefErrorDialog();
		} finally {
			ser.close();
			try {
				out.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	private void savePrefErrorDialog() {
		MessageBox errorBox = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
		errorBox.setText("Error");
		errorBox.setMessage("Cannot save the playlist information");
		errorBox.open();
	}

	private void readPrefErrorDialog() {
		MessageBox errorBox = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
		errorBox.setText("Error");
		errorBox.setMessage("Cannot read the playlist information");
		errorBox.open();
	}

	@SuppressWarnings("unchecked")
	public void loadPlaylists() {
		String value = preferences.get(prefPlaylistKey, "");
		byte bytedata[] = value.getBytes();
		ByteArrayInputStream bais = new ByteArrayInputStream(bytedata);
		DeSerializer ds;
		try {
			ds = new DeSerializer(bais);
			HashMap<String, String> data = (HashMap<String, String>) ds.get();
			if (data != null) {
				playlistsPaths = data;
			}
		} catch (Exception e) {
			e.printStackTrace();
			readPrefErrorDialog();
			try {
				preferences.clear();
				preferences.sync();
			} catch (BackingStoreException e1) {
				e1.printStackTrace();
			}

		}
	}

	public Freezer() {
		display = new Display();
		shell = new Shell(display);



		preferences = Preferences.userNodeForPackage(this.getClass());

		playlistsPaths = new HashMap<String, String>();

		loadPlaylists();

		initTray();

		updatePlaylistMenu();

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		trayIcon.dispose();
		display.dispose();
	}

	public void setPlaylistsPaths(HashMap<String, String> paths) {
		this.playlistsPaths = paths;
	}

	public static void main(String[] args) {
		new Freezer();
	}

	public HashMap<String, String> getPlaylistsPaths() {
		return playlistsPaths;
	}

	public String getPlaylistPath(String key) {
		if (playlistsPaths.containsKey(key)) {
			return playlistsPaths.get(key);
		} else {
			return "";
		}
	}

	public void removePlaylist(String key) {
		if (playlistsPaths.containsKey(key)) {
			playlistsPaths.remove(key);
			updatePlaylistMenu();
		}
	}

	public boolean existPlaylist(String key) {
		return playlistsPaths.containsKey(key);
	}

	public void setPath(String key, String path) {
		playlistsPaths.put(key, path);
		savePlaylists();
		updatePlaylistMenu();
	}

	public String getPath(String key) {
		if (existPlaylist(key))
			return playlistsPaths.get(key);
		else
			return "";
	}

}
