/**
    FREEZER - A tray icon for Deezer

    Copyright (C) 2007 Amos Brocco <amos.brocco@unifr.ch>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
import java.util.Iterator;
import java.util.Set;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class PlaylistManager {

	private Shell shell;
	private int WIDTH = 480;
	private int HEIGHT = 320;
	private org.eclipse.swt.widgets.List pllist;
	private Freezer freezer;

	public PlaylistManager(Freezer fr, Display display) {
		freezer = fr;
		shell = new Shell(display, SWT.TITLE | SWT.BORDER
				| SWT.APPLICATION_MODAL);
		shell.setSize(WIDTH, HEIGHT);
		shell.setText("Manage playlists");
		int screenWidth = Display.getCurrent().getClientArea().width;
		int screenHeight = Display.getCurrent().getClientArea().height;
		int x = (int) (screenWidth / 2 - (WIDTH / 2));
		int y = (int) (screenHeight / 2 - (HEIGHT / 2));
		shell.setLocation(x, y);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 4;
		gridLayout.makeColumnsEqualWidth = false;
		shell.setLayout(gridLayout);

		Label label = new Label(shell, SWT.WRAP);
		label.setText("Playlists:");
		GridData labelgridData = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
		labelgridData.horizontalSpan = 4;
		labelgridData.horizontalAlignment = GridData.FILL;
		label.setLayoutData(labelgridData);

		pllist = new org.eclipse.swt.widgets.List(shell, SWT.BORDER
				| SWT.V_SCROLL);
		pllist.setSize(pllist.getSize().x, 200);
		GridData pllistgridData = new GridData(GridData.VERTICAL_ALIGN_CENTER);
		pllistgridData.horizontalSpan = 4;
		pllistgridData.grabExcessVerticalSpace = true;
		pllistgridData.grabExcessHorizontalSpace = true;
		pllistgridData.horizontalAlignment = GridData.FILL;
		pllistgridData.verticalAlignment = GridData.FILL;
		pllist.setLayoutData(pllistgridData);

		Button addBtn = new Button(shell, SWT.PUSH);
		addBtn.setText("New playlist");
		addBtn.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING));
		addBtn.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent event) {
				addNew();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

		Button removeBtn = new Button(shell, SWT.PUSH);
		removeBtn.setText("Delete selected");
		removeBtn.setLayoutData(new GridData(
				GridData.HORIZONTAL_ALIGN_BEGINNING));
		removeBtn.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent event) {
				removeSelected();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

		Button modifyBtn = new Button(shell, SWT.PUSH);
		modifyBtn.setText("Modify path");
		modifyBtn.setLayoutData(new GridData(
				GridData.HORIZONTAL_ALIGN_BEGINNING));
		modifyBtn.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent event) {
				modifySelected();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

		Button closeBtn = new Button(shell, SWT.PUSH);
		closeBtn.setText("Close");
		closeBtn.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		closeBtn.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent event) {
				shell.dispose();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

	}

	private void addNew() {
		final Shell addDialog = new Shell(shell, SWT.TITLE | SWT.BORDER
				| SWT.APPLICATION_MODAL);
		addDialog.setMinimumSize(320, 0);
		addDialog.setLayout(new GridLayout(2, false));
		addDialog.setText("Add new playlist");

		Label label = new Label(addDialog, SWT.NULL);
		label.setText("Playlist name:");
		GridData labeldata = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
		labeldata.horizontalSpan = 2;
		labeldata.grabExcessVerticalSpace = false;
		labeldata.grabExcessHorizontalSpace = true;
		labeldata.horizontalAlignment = GridData.FILL;
		labeldata.verticalAlignment = GridData.FILL;
		label.setLayoutData(labeldata);

		final Text name = new Text(addDialog, SWT.SINGLE | SWT.BORDER);
		GridData namedata = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
		namedata.horizontalSpan = 2;
		namedata.grabExcessVerticalSpace = false;
		namedata.grabExcessHorizontalSpace = true;
		namedata.horizontalAlignment = GridData.FILL;
		namedata.verticalAlignment = GridData.FILL;
		name.setLayoutData(namedata);

		Button doAdd = new Button(addDialog, SWT.PUSH);
		doAdd.setText("Add");
		doAdd.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING));

		Button doCancel = new Button(addDialog, SWT.PUSH);
		doCancel.setText("Cancel");
		doCancel
				.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING));

		doAdd.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				String plname = name.getText().trim();
				if (!freezer.existPlaylist(plname)) {
					pllist.add(plname);
					freezer.setPath(plname, "");
					addDialog.dispose();
					pllist.select(pllist.indexOf(plname));
					modifySelected();
				}
			}
		});

		doCancel.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				addDialog.dispose();
			}
		});

		addDialog.addListener(SWT.Traverse, new Listener() {
			public void handleEvent(Event event) {
				if (event.detail == SWT.TRAVERSE_ESCAPE) {
					addDialog.dispose();
				}
			}
		});

		addDialog.pack();
		
		int screenWidth = Display.getCurrent().getClientArea().width;
		int screenHeight = Display.getCurrent().getClientArea().height;
		int x = (int) (screenWidth / 2 - (addDialog.getSize().x / 2));
		int y = (int) (screenHeight / 2 - (addDialog.getSize().y / 2));
		addDialog.setLocation(x, y);
		
		addDialog.open();

		Display display = shell.getDisplay();
		while (!addDialog.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
	}

	private void removeSelected() {
		int selected = pllist.getSelectionIndex();
		if (selected >= 0) {
			final String key = pllist.getItem(selected);
			freezer.removePlaylist(key);
			pllist.remove(selected);
		}
	}

	private void modifySelected() {
		int selected = pllist.getSelectionIndex();
		if (selected >= 0) {
			final String key = pllist.getItem(selected);
			String currentPath = freezer.getPath(key);

			final Shell modifyDialog = new Shell(shell, SWT.TITLE | SWT.BORDER
					| SWT.APPLICATION_MODAL);
			modifyDialog.setMinimumSize(320, 0);
			modifyDialog.setLayout(new GridLayout(2, false));
			modifyDialog.setText(String.format("Modify playlist %s", key));

			Label label = new Label(modifyDialog, SWT.NULL);
			label.setText("Deezer path:");
			GridData labeldata = new GridData(
					GridData.HORIZONTAL_ALIGN_BEGINNING);
			labeldata.horizontalSpan = 2;
			labeldata.grabExcessVerticalSpace = false;
			labeldata.grabExcessHorizontalSpace = true;
			labeldata.horizontalAlignment = GridData.FILL;
			labeldata.verticalAlignment = GridData.FILL;
			label.setLayoutData(labeldata);

			final Text path = new Text(modifyDialog, SWT.SINGLE | SWT.BORDER);
			path.setText(currentPath);
			GridData pathdata = new GridData(
					GridData.HORIZONTAL_ALIGN_BEGINNING);
			pathdata.horizontalSpan = 2;
			pathdata.grabExcessVerticalSpace = false;
			pathdata.grabExcessHorizontalSpace = true;
			pathdata.horizontalAlignment = GridData.FILL;
			pathdata.verticalAlignment = GridData.FILL;
			path.setLayoutData(pathdata);

			Button doModify = new Button(modifyDialog, SWT.PUSH);
			doModify.setText("Save");
			doModify.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));

			Button doCancel = new Button(modifyDialog, SWT.PUSH);
			doCancel.setText("Cancel");

			doModify.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					freezer.setPath(key, path.getText());
					modifyDialog.dispose();
				}
			});

			doCancel.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					modifyDialog.dispose();
				}
			});

			modifyDialog.addListener(SWT.Traverse, new Listener() {
				public void handleEvent(Event event) {
					if (event.detail == SWT.TRAVERSE_ESCAPE) {
						modifyDialog.dispose();
					}
				}
			});

			modifyDialog.pack();
			
			int screenWidth = Display.getCurrent().getClientArea().width;
			int screenHeight = Display.getCurrent().getClientArea().height;
			int x = (int) (screenWidth / 2 - (modifyDialog.getSize().x / 2));
			int y = (int) (screenHeight / 2 - (modifyDialog.getSize().y / 2));
			modifyDialog.setLocation(x, y);
			
			modifyDialog.open();

			Display display = shell.getDisplay();
			while (!modifyDialog.isDisposed()) {
				if (!display.readAndDispatch())
					display.sleep();
			}
		}
	}

	public void populate(Set<String> playlists) {
		pllist.removeAll();
		Iterator<String> iter = playlists.iterator();
		while (iter.hasNext()) {
			pllist.add(iter.next());
		}
	}

	public void setVisible(boolean b) {
		shell.setVisible(b);
	}

	public void show() {
		shell.setVisible(true);
		shell.open();
		Display display = shell.getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
	}

}
